@@ -17,10 +17,7 @@
 
 struct RefAny<'a, T>(&'a T);
 
-impl<#[rustc_1] 'a, #[rustc_2] T, #[oops]> RefAny<'a, T> {
-    //~^ ERROR expected identifier, found `>`
-}
+impl<#[rustc_1] 'a, #[rustc_2] T, #[oops]> RefAny<'a, T> {}
+//~^ ERROR trailing attribute after type parameters
 
-fn main() {
-
-}
+fn main() {}
